/*
  * Copyright 2012  Samsung Electronics Co., Ltd
  *
  * Licensed under the Flora License, Version 1.1 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *     http://floralicense.org/license/
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
/**
 * @file    permission_delete_view.cpp
 * @author  Leerang Song (leerang.song@samsung.com)
 */


#include "permission_delete_view.h"

#include <memory>
#include <string>
#include <Elementary.h>
#include <dpl/assert.h>
#include <dpl/foreach.h>
#include <wrt-commons/security-origin-dao/security_origin_dao.h>

#include "util.h"
#include "context.h"
#include "menu_data.h"
#include "wrt_setting_text.h"

using namespace SecurityOriginDB;

namespace WebAppDetailSetting {
namespace {
// ETC
const char* const APP_SCHEME_ORIGIN = "local";
const unsigned int TOOLBAR_LEFT_INDEX = 0;
const unsigned int TOOLBAR_RIGHT_INDEX = 1;
} // anonymous namespace

PermissionDeleteView::PermissionDeleteView(Context& context) :
    m_context(context)
{
}

PermissionDeleteView::~PermissionDeleteView(void)
{
}

void PermissionDeleteView::init(void)
{
    // init genlist
    loadView();
}

void PermissionDeleteView::loadView(void)
{
    Assert(m_context.parent);

    // Create Genlist
    m_genlist = createGenlist(m_context.parent);

    // push to parent(naviframe)
    Elm_Object_Item* objectItem =
        elm_naviframe_item_push(m_context.parent,
                                m_context.selectedOrigin.c_str(),
                                NULL,
                                NULL,
                                m_genlist,
                                NULL);

    // Create toolbar
    Evas_Object* toolbar = createToolbar(m_context.parent);

    // set toolbar to Elm_Object_Item
    elm_object_item_part_content_set(objectItem, "toolbar", toolbar);
}

Evas_Object* PermissionDeleteView::createGenlist(Evas_Object* parent)
{
    Assert(parent);
    Evas_Object* genlist = elm_genlist_add(parent);
    elm_genlist_block_count_set(genlist, m_context.permissionData.size());

    unsigned int index = 0;
    FOREACH(it, m_context.permissionData) {
        Elm_Genlist_Item_Class* itc = elm_genlist_item_class_new();
        itc->item_style = "2text.2icon.4";
        itc->func.text_get = getTextCallback;
        itc->func.content_get = getContentCallback;
        itc->func.state_get = NULL;
        itc->func.del = delCallback;

        GenlistData* data = new GenlistData(index++, static_cast<void*>(this));
        Elm_Object_Item* objectItem =
            elm_genlist_item_append(genlist,
                                    itc,
                                    static_cast<void*>(data),
                                    NULL,
                                    ELM_GENLIST_ITEM_NONE,
                                    NULL,
                                    NULL);
        data->item = objectItem;
        elm_genlist_item_class_free(itc);
        if (m_context.dao->isReadOnly(*(*it))) {
            elm_object_item_disabled_set(objectItem, EINA_TRUE);
        }
        elm_genlist_item_select_mode_set(objectItem,
                                         ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
    }
    return genlist;
}

char* PermissionDeleteView::getTextCallback(void* data,
                                            Evas_Object* obj,
                                            const char* part)
{
    UNUSED_PARAM(obj);

    // get PermissionView instance
    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    PermissionDeleteView* This =
        static_cast<PermissionDeleteView*>(glData->data);
    Assert(This);

    // get SecurityOriginDataPtr based current index
    SecurityOriginDataPtr originData = This->m_context.permissionData[glData->index];

    // search text based feature index
    Assert(part);
    if (ElmPart::TEXT1 == part) {
        return WrtSettingText::getFeatureText(originData->feature);
    } else if (ElmPart::TEXT2 == part) {
        return WrtSettingText::getDescriptionText(originData->feature);
    }
    return strdup("Unknown");
}

Evas_Object* PermissionDeleteView::getContentCallback(void* data,
                                                      Evas_Object* obj,
                                                      const char* part)
{
    // get PermissionView instance
    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    PermissionDeleteView* This =
        static_cast<PermissionDeleteView*>(glData->data);
    Assert(This);
    Context& context = This->m_context;

    if (ElmPart::ICON1 == part) {
        // create check
        Assert(obj);
        Evas_Object* check = This->createDefaultCheck(obj);
        evas_object_smart_callback_add(check,
                                       "changed",
                                       checkChangedCallback,
                                       data);
        return check;
    } else if (ElmPart::ICON2 == part) {
        // create check
        Assert(obj);
        Evas_Object* check = This->createOnOffCheck(obj);

        // get result of permission
        bool result = context.getPermissionResult(glData->index);

        // set check state
        elm_check_state_set(check, result ? EINA_TRUE : EINA_FALSE);
        return check;
    }

    _W("Uknown part: %s", part);
    return NULL;
}

void PermissionDeleteView::delCallback(void* data, Evas_Object* obj)
{
    UNUSED_PARAM(obj);

    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    delete glData;
}

Evas_Object* PermissionDeleteView::createDefaultCheck(Evas_Object* parent)
{
    Evas_Object* check = elm_check_add(parent);
    elm_object_style_set(check, "default");
    elm_check_state_set(check, EINA_FALSE);
    return check;
}

Evas_Object* PermissionDeleteView::createOnOffCheck(Evas_Object* parent)
{
    Evas_Object* check = elm_check_add(parent);
    evas_object_propagate_events_set(check, EINA_FALSE);
    elm_object_style_set(check, "on&off");
    elm_object_disabled_set(check,  EINA_TRUE);
    return check;
}

void PermissionDeleteView::checkChangedCallback(void* data, Evas_Object* obj, void* eventInfo)
{
    UNUSED_PARAM(eventInfo);

    Assert(obj);
	Eina_Bool state = elm_check_state_get(obj);

    Assert(data);
    GenlistData* glData = static_cast<GenlistData*>(data);
    glData->check = state;
}

Evas_Object* PermissionDeleteView::createToolbar(Evas_Object* parent)
{
    Assert(parent);
    Evas_Object* toolbar = elm_toolbar_add(parent);
    elm_object_style_set(toolbar, "default");
    elm_toolbar_shrink_mode_set(toolbar, ELM_TOOLBAR_SHRINK_EXPAND);
    elm_toolbar_transverse_expanded_set(toolbar, EINA_TRUE);
    elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_NONE);

    ToolbarData* tbData =
        new ToolbarData(TOOLBAR_LEFT_INDEX,
                        static_cast<void*>(this));
    // left button "Delete"
    Elm_Object_Item* item =
        elm_toolbar_item_append(toolbar,
                                NULL,
                                WRT_SETTING_SK_DELETE,
                                toolbarClickedCallback,
                                static_cast<void*>(tbData));
    elm_object_item_del_cb_set(item, toolbarDelCallback);

    tbData = new ToolbarData(TOOLBAR_RIGHT_INDEX, static_cast<void*>(this));
    // right button "Cancel"
    item = elm_toolbar_item_append(toolbar,
                                   NULL,
                                   WRT_SETTING_SK_CANCEL,
                                   toolbarClickedCallback,
                                   static_cast<void*>(tbData));
    elm_object_item_del_cb_set(item, toolbarDelCallback);

    return toolbar;
}

void PermissionDeleteView::toolbarClickedCallback(void* data, Evas_Object* obj, void* eventInfo)
{
    UNUSED_PARAM(obj);
    UNUSED_PARAM(eventInfo);

    Assert(data);
    ToolbarData* tbData = static_cast<ToolbarData*>(data);
    PermissionDeleteView* This =
        static_cast<PermissionDeleteView*>(tbData->data);
    Assert(This);
    Context& context = This->m_context;

    if (tbData->index == TOOLBAR_LEFT_INDEX) {
        // "Delete" - back to previous menu
        Elm_Object_Item* item = elm_genlist_first_item_get(This->m_genlist);
        while (item) {
            GenlistData* glData =
                static_cast<GenlistData*>(elm_object_item_data_get(item));
            if (glData->check) {
                context.removePermissionData(glData->index);
            }
            item = elm_genlist_item_next_get(item);
        }
        // Update previous menu
        context.cbs.originViewUpdate();
        context.cbs.permissionViewUpdate();
        elm_naviframe_item_pop(context.parent);
    } else if (tbData->index == TOOLBAR_RIGHT_INDEX) {
        // "Cancel" - back to previous menu
        elm_naviframe_item_pop(context.parent);
    }
}

void PermissionDeleteView::toolbarDelCallback(void* data, Evas_Object* obj, void* eventInfo)
{
    UNUSED_PARAM(obj);
    UNUSED_PARAM(eventInfo);

    Assert(data);
    ToolbarData* tbData = static_cast<ToolbarData*>(data);
    delete tbData;
}
} // namespace WebAppDetailSetting
